#include "libMath.h"

static void psdes(uint32_t * lword, uint32_t * irword);

static void qsort1(const size_t n, float arr[]);
static void qsort2(const size_t n, float arr[], float brr[]);
static void crank(const size_t n, float w[], float s[]);

static void shift3(float *a, float *b, float *c, float *d);

double powint(double x, int n)
{
 int i;
 double y = 1.0;

 switch (n) {
 case 2:
  return x * x;
  break;

 case 1:
  return x;
  break;

 case 0:
  return 1.0;
  break;

 case -1:
  return 1.0 / x;
  break;

 case -2:
  return 1.0 / x / x;
  break;

 case 3:
 case 6:
  y *= x * x * x;
  if (n == 3)
   return y;
  else
   return y * y;
  break;

 case -3:
 case -6:
  y /= x * x * x;
  if (n == -3)
   return y;
  else
   return y * y;
  break;

 default:
  if (abs(n) > 32)
   return pow(x, (double) (n));

  else if (n > 0) {
   for (i = 1; i <= n; i++)
    y *= x;
  } else {
   for (i = 1; i <= -n; i++)
    y /= x;
  }
 }

 return y;
}

float gasdev(int32_t idum)
{
 static int iset = 0;
 static float gset;
 float fac, rsq, v1, v2;

 if (iset == 0) {

  do {
   v1 = 2.0 * ran1(idum) - 1.0;
   v2 = 2.0 * ran1(idum) - 1.0;

   rsq = v1 * v1 + v2 * v2;
  }
  while (rsq >= 1.0 || rsq == 0.0);

  fac = sqrt(-2.0 * log(rsq) / rsq);

  gset = v1 * fac;
  iset = 1;

  return v2 * fac;
 }

 else {
  iset = 0;

  return gset;
 }
}

#define NTAB	32

static int32_t _idum = -1;
static int32_t iy = 0;
static int32_t iv[NTAB];

float ran1(int32_t idum)
{
 const int32_t IA = 16807, IM = 2147483647, IQ = 127773, IR = 2836;
 const int32_t NDIV = (1 + (IM - 1) / NTAB);
 const float EPS = 2.22e-16, AM = 1.0 / IM, RNMX = (1.0 - EPS);
 int j, k;
 float temp;

 if (_idum <= 0 || !iy) {
  if (-_idum < 1)
   _idum = 1;
  else
   _idum = -_idum;

  for (j = NTAB + 7; j >= 0; j--) {
   k = _idum / IQ;
   _idum = IA * (_idum - k * IQ) - IR * k;

   if (_idum < 0)
    _idum += IM;

   if (j < NTAB)
    iv[j] = _idum;
  }

  iy = iv[0];
 }

 k = _idum / IQ;
 _idum = IA * (_idum - k * IQ) - IR * k;

 if (_idum < 0)
  _idum += IM;

 j = iy / NDIV;

 iy = iv[j];
 iv[j] = _idum;

 if ((temp = AM * iy) > RNMX)
  return RNMX;
 else
  return temp;
}

static uint32_t idums = 0, idumr = 0;

#if defined(vax) || defined(_vax_) || defined(__vax__) || defined(VAX)
static const uint32_t jflone = 0x00004080;
static const uint32_t jflmsk = 0xffff007f;
#else
static const uint32_t jflone = 0x3f800000;
static const uint32_t jflmsk = 0x007fffff;
#endif

float ran4(int32_t idum)
{
 uint32_t irword, lword, itemp = 0;

 if (idum < 0) {
  idums = -idum;
  idumr = 0;
 }

 else {
  irword = idumr + idum;
  lword = idums;

  psdes(&lword, &irword);

  itemp = jflone | (jflmsk & irword);
 }

 ++idumr;

 return (*(float *) (void *) (&itemp)) - 1.0;
}

#define NITER	4

static const uint32_t c1[NITER] = {
 0xbaa96887, 0x1e17d32c, 0x03bcdc3c, 0x0f33d1b2
};
static const uint32_t c2[NITER] = {
 0x4b0f3b58, 0xe874f0c3, 0x6955c5a6, 0x55a7ca46
};

static void psdes(uint32_t * lword, uint32_t * irword)
{
 int i;
 uint32_t ia, ib, iswap, itmph = 0, itmpl = 0;

 for (i = 0; i < NITER; i++) {
  ia = (iswap = *irword) ^ c1[i];
  itmpl = ia & 0xffff;
  itmph = ia >> 16;
  ib = itmpl * itmpl + ~(itmph * itmph);
  *irword = *lword ^ (((ia = (ib >> 16) | ((ib & 0xffff) << 16)) ^ c2[i]) + itmpl * itmph);
  *lword = iswap;
 }
}

static const double cof[6] = { 76.18009172947146, -86.50532032941677,
 24.01409824083091, -1.231739572450155, 0.1208650973866179e-2, -0.5395239384953e-5
};

float gammln(const float xx)
{
 double x, y, tmp, ser;
 int j;

 y = x = xx;
 tmp = x + 5.5;
 tmp -= (x + 0.5) * log(tmp);
 ser = 1.000000000190015;

 for (j = 0; j < 6; j++)
  ser += cof[j] / ++y;

 return -tmp + log(2.5066282746310005 * ser / x);
}

float factln(const int n)
{
 static float a[101];

 if (n < 0) {
  fprintf(stderr, "Negative factorial in factln().\n");
  exit(EXIT_FAILURE);
 }

 if (n <= 1)
  return 0.0;

 if (n <= 100)
  return (a[n] != 0.0 ? a[n] : (a[n] = gammln(n + 1.0)));
 else
  return gammln(n + 1.0);
}

float factrl(const int n)
{
 static int ntop = 4;
 static float a[33] = { 1.0, 1.0, 2.0, 6.0, 24.0 };
 int j;

 if (n < 0) {
  fprintf(stderr, "Negative factorial in factrl().\n");
  exit(EXIT_FAILURE);
 }

 if (n > 32)
  return exp(gammln(n + 1.0));

 while (ntop < n) {
  j = ntop++;
  a[ntop] = a[j] * ntop;
 }

 return a[n];
}

float perm(int n, int k)
{
 return floor(0.5 + exp(factln(n) - factln(k)));
}

float bico(int n, int k)
{
 return floor(0.5 + exp(factln(n) - factln(k) - factln(n - k)));
}

float erfcc(float x)
{
 float t, z, ans;

 z = fabs(x);
 t = 1.0 / (1.0 + 0.5 * z);

 ans =
     t * exp(-z * z - 1.26551223 + t * (1.00002368 + t * (0.37409196 + t * (0.09678418 + t * (-0.18628806 +
		     t * (0.27886807 + t * (-1.13520398 + t * (1.48851587 + t * (-0.82215223 + t * 0.17087277)))))))));

 return x >= 0.0 ? ans : 2.0 - ans;
}

float betai(float a, float b, float x)
{
 float bt;

 if (x < 0.0 || x > 1.0) {
  fprintf(stderr, "Bad x in betai().\n");
  exit(EXIT_FAILURE);
 }

 if (x == 0.0 || x == 1.0)
  bt = 0.0;
 else
  bt = exp(gammln(a + b) - gammln(a) - gammln(b) + a * log(x) + b * log(1.0 - x));

 if (x < (a + 1.0) / (a + b + 2.0))
  return bt * betacf(a, b, x) / a;
 else
  return 1.0 - bt * betacf(b, a, 1.0 - x) / b;
}

float betacf(float a, float b, float x)
{
 const int MAXIT = 100;
 const double EPS = 2.22e-16;
 const double FPMIN = 2.23e-308 / EPS;
 int m, m2;
 double aa, c, d, del, h, qab, qam, qap;

 qab = a + b;
 qap = a + 1.0;
 qam = a - 1.0;
 c = 1.0;
 d = 1.0 - qab * x / qap;

 if (fabs(d) < FPMIN)
  d = FPMIN;

 d = 1.0 / d;
 h = d;

 for (m = 1; m <= MAXIT; m++) {
  m2 = 2 * m;
  aa = m * (b - m) * x / ((qam + m2) * (a + m2));
  d = 1.0 + aa * d;

  if (fabs(d) < FPMIN)
   d = FPMIN;

  c = 1.0 + aa / c;

  if (fabs(c) < FPMIN)
   c = FPMIN;

  d = 1.0 / d;
  h *= d * c;
  aa = -(a + m) * (qab + m) * x / ((a + m2) * (qap + m2));
  d = 1.0 + aa * d;

  if (fabs(d) < FPMIN)
   d = FPMIN;

  c = 1.0 + aa / c;

  if (fabs(c) < FPMIN)
   c = FPMIN;

  d = 1.0 / d;
  del = d * c;
  h *= del;

  if (fabs(del - 1.0) <= EPS)
   break;
 }

 if (m > MAXIT) {
  fprintf(stderr, "a or b is too big, or MAXIT is too small in betacf().\n");
  exit(EXIT_FAILURE);
 }

 return h;
}

static void qsort1(const size_t n, float arr[])
{
 const int M = 7, NSTACK = 50;
 int i, ir, j, k, jstack = -1, l = 0, li;
 float a, dum;
 int istack[NSTACK];
/*
 istack = (int *) (malloc(NSTACK * sizeof(int)));

 if (istack == NULL) {
  fprintf(stderr, "Short of memory in qsort1().\n");
  exit(EXIT_FAILURE);
 }
*/
 ir = n - 1;

 for (;;) {
  if (ir - l < M) {
   for (j = l + 1; j <= ir; j++) {
    a = arr[j];

    for (i = j - 1; i >= l; i--) {

     if (arr[i] <= a)
      break;

     arr[i + 1] = arr[i];
    }

    arr[i + 1] = a;
   }

   if (jstack < 0)
    break;

   ir = istack[jstack--];
   l = istack[jstack--];
  } else {
   k = (l + ir) >> 1;
   li = l + 1;

   dum = arr[k];
   arr[k] = arr[li];
   arr[li] = dum;

   if (arr[l] > arr[ir]) {
    dum = arr[l];
    arr[l] = arr[ir];
    arr[ir] = dum;
   }

   if (arr[li] > arr[ir]) {
    dum = arr[li];
    arr[li] = arr[ir];
    arr[ir] = dum;
   }

   if (arr[l] > arr[li]) {
    dum = arr[l];
    arr[l] = arr[li];
    arr[li] = dum;
   }

   i = li;
   j = ir;

   a = arr[li];

   for (;;) {
    do
     i++;
    while (arr[i] < a);

    do
     j--;
    while (arr[j] > a);

    if (j < i)
     break;

    dum = arr[i];
    arr[i] = arr[j];
    arr[j] = dum;
   }

   arr[li] = arr[j];
   arr[j] = a;

   jstack += 2;

   if (jstack >= NSTACK) {
    fprintf(stderr, "NSTACK is too small in qsort1().\n");
    exit(EXIT_FAILURE);
   }

   if (ir - i + 1 >= j - l) {
    istack[jstack] = ir;
    istack[jstack - 1] = i;
    ir = j - 1;
   } else {
    istack[jstack] = j - 1;
    istack[jstack - 1] = l;
    l = i;
   }
  }
 }
/*
 free((char *) (istack));
*/
}

static void qsort2(const size_t n, float arr[], float brr[])
{
 const int M = 7, NSTACK = 50;
 int i, ir, j, k, jstack = -1, l = 0, li;
 float a, b, dum;
 int istack[NSTACK];
/*
 istack = (int *) (malloc(NSTACK * sizeof(int)));

 if (istack == NULL) {
  fprintf(stderr, "Short of memory in qsort2().\n");
  exit(EXIT_FAILURE);
 }
*/
 ir = n - 1;

 for (;;) {
  if (ir - l < M) {
   for (j = l + 1; j <= ir; j++) {
    a = arr[j];
    b = brr[j];

    for (i = j - 1; i >= l; i--) {

     if (arr[i] <= a)
      break;

     arr[i + 1] = arr[i];
     brr[i + 1] = brr[i];
    }

    arr[i + 1] = a;
    brr[i + 1] = b;
   }

   if (jstack < 0)
    break;

   ir = istack[jstack--];
   l = istack[jstack--];
  } else {
   k = (l + ir) >> 1;
   li = l + 1;

   dum = arr[k];
   arr[k] = arr[li];
   arr[li] = dum;

   dum = brr[k];
   brr[k] = brr[li];
   brr[li] = dum;

   if (arr[l] > arr[ir]) {
    dum = arr[l];
    arr[l] = arr[ir];
    arr[ir] = dum;

    dum = brr[l];
    brr[l] = brr[ir];
    brr[ir] = dum;
   }

   if (arr[li] > arr[ir]) {
    dum = arr[li];
    arr[li] = arr[ir];
    arr[ir] = dum;

    dum = brr[li];
    brr[li] = brr[ir];
    brr[ir] = dum;
   }

   if (arr[l] > arr[li]) {
    dum = arr[l];
    arr[l] = arr[li];
    arr[li] = dum;

    dum = brr[l];
    brr[l] = brr[li];
    brr[li] = dum;
   }

   i = li;
   j = ir;

   a = arr[li];
   b = brr[li];

   for (;;) {
    do
     i++;
    while (arr[i] < a);

    do
     j--;
    while (arr[j] > a);

    if (j < i)
     break;

    dum = arr[i];
    arr[i] = arr[j];
    arr[j] = dum;

    dum = brr[i];
    brr[i] = brr[j];
    brr[j] = dum;
   }

   arr[li] = arr[j];
   arr[j] = a;
   brr[li] = brr[j];
   brr[j] = b;

   jstack += 2;

   if (jstack >= NSTACK) {
    fprintf(stderr, "NSTACK is too small in qsort2().\n");
    exit(EXIT_FAILURE);
   }

   if (ir - i + 1 >= j - l) {
    istack[jstack] = ir;
    istack[jstack - 1] = i;
    ir = j - 1;
   } else {
    istack[jstack] = j - 1;
    istack[jstack - 1] = l;
    l = i;
   }
  }
 }
/*
 free((char *) (istack));
*/
}

static void crank(const size_t n, float w[], float *s)
{
 int j = 1, ji, jt;
 float t, rank;

 *s = 0.0;

 while (j < n) {
  if (w[j] != w[j - 1]) {
   w[j - 1] = j;
   ++j;
  } else {
   for (jt = j + 1; jt <= n && w[jt - 1] == w[j - 1]; jt++);
   rank = 0.5 * (j + jt - 1);

   for (ji = j; ji <= jt - 1; ji++)
    w[ji - 1] = rank;

   t = jt - j;
   *s += t * (t + 1.0) * (t - 1.0);
   j = jt;
  }
 }

 if (j == n)
  w[n - 1] = n;
}

/* linear correlation coefficient (Pearson's rho) */

void pearsn(float x[], float y[], const size_t n, float *r, float *z, float *prob)
{
 const double TINY = 1.0e-20;
 int j;
 double yt, xt, t, df;
 double syy = 0.0, sxy = 0.0, sxx = 0.0, ay = 0.0, ax = 0.0;

 for (j = 0; j < n; j++) {
  ax += x[j];
  ay += y[j];
 }

 ax /= n;
 ay /= n;

 for (j = 0; j < n; j++) {
  xt = x[j] - ax;
  yt = y[j] - ay;

  sxx += xt * xt;
  syy += yt * yt;
  sxy += xt * yt;
 }

 *r = sxy / (sqrt(sxx * syy) + TINY);

 if (z == NULL)
  return;

 *z = 0.5 * log((1.0 + (*r) + TINY) / (1.0 - (*r) + TINY));

 if (prob == NULL)
  return;

 df = n - 2;
 t = (*r) * sqrt(df / ((1.0 - (*r) + TINY) * (1.0 + (*r) + TINY)));

 *prob = betai(0.5 * df, 0.5, df / (df + t * t));
}

/* rank correlation coefficient (Spearman's r) */

#define WKSP_SIZE	128

static float _wksp1[WKSP_SIZE], _wksp2[WKSP_SIZE];

void spear(float x[], float y[], const size_t n, float *rs, float *zd, float *probrs, float *d, float *probd)
{
 int j;
 float vard, t, sg, sf, fac, en3n, en, df, aved;
 float *wksp1 = NULL, *wksp2 = NULL;

 if (n <= WKSP_SIZE) {
  wksp1 = _wksp1;
  wksp2 = _wksp2;
 }

 else {
  wksp1 = (float *) (malloc(n * sizeof(float)));

  if (wksp1 == NULL) {
   fprintf(stderr, "Short of memory in spear().\n");
   exit(EXIT_FAILURE);
  }

  wksp2 = (float *) (malloc(n * sizeof(float)));

  if (wksp2 == NULL) {
   fprintf(stderr, "Short of memory in spear().\n");
   exit(EXIT_FAILURE);
  }
 }

 for (j = 0; j < n; j++) {
  wksp1[j] = x[j];
  wksp2[j] = y[j];
 }

 qsort2(n, wksp1, wksp2);
 crank(n, wksp1, &sf);
 qsort2(n, wksp2, wksp1);
 crank(n, wksp2, &sg);

 *d = 0.0;
 for (j = 0; j < n; j++)
  (*d) += powint(wksp1[j] - wksp2[j], 2);

 en = n;
 en3n = en * (en + 1.0) * (en - 1.0);
 aved = en3n / 6.0 - (sf + sg) / 12.0;
 fac = (1.0 - sf / en3n) * (1.0 - sg / en3n);
 vard = ((en - 1.0) * en * en * powint(en + 1.0, 2) / 36.0) * fac;

 *rs = (1.0 - (6.0 / en3n) * ((*d) + (sf + sg) / 12.0)) / sqrt(fac);

 if (zd == NULL)
  goto escape;

 *zd = ((*d) - aved) / sqrt(vard);
 *probd = erfcc(fabs((*zd)) / M_SQRT2);

 if (probrs == NULL)
  goto escape;

 fac = ((*rs) + 1.0) * (1.0 - (*rs));

 if (fac > 0.0) {
  t = (*rs) * sqrt((en - 2.0) / fac);
  df = en - 2.0;
  *probrs = betai(0.5 * df, 0.5, df / (df + t * t));
 } else
  *probrs = 0.0;

escape:if (n <= WKSP_SIZE)
  return;

 free((char *) (wksp1));
 free((char *) (wksp2));
}

/* rank correlation coefficient (Kendall's tau) */

void kendl1(float x[], float y[], const size_t n, float *tau, float *z, float *prob)
{
 int is = 0, j, k, n2 = 0, n1 = 0;
 float svar, aa, a2, a1;

 for (j = 0; j < n - 1; j++) {
  for (k = j + 1; k < n; k++) {
   a1 = x[j] - x[k];
   a2 = y[j] - y[k];
   aa = a1 * a2;

   if (aa != 0.0) {
    ++n1;
    ++n2;
    aa > 0.0 ? ++is : --is;
   } else {
    if (a1 != 0.0)
     ++n1;
    if (a2 != 0.0)
     ++n2;
   }
  }
 }

 if (n1 == n2)
  *tau = is / (float) (n1);
 else
  *tau = is / (sqrt((double) (n1)) * sqrt((double) (n2)));

 if (z == NULL)
  return;

 svar = (4.0 * n + 10.0) / (9.0 * n * (n - 1.0));
 *z = (*tau) / sqrt(svar);

 if (prob == NULL)
  return;

 *prob = erfcc(fabs(*z) / M_SQRT2);
}

/* Geni's coefficient */

void gini(float x[], const size_t n, float *gc)
{
 int i;
 float d, g, t, s, tot = 0.0;

 if (n < 2) {
  *gc = 1.0;
  return;
 }

 qsort1(n, x);

 for (i = 0; i < n; i++)
  tot += x[i];

 t = tot * (float) (n - 1) / 2.0;

 g = tot / (float) (n);

 d = s = 0.0;

 for (i = 0; i < n - 1; i++) {
  s += x[i];
  d += g * (float) (i + 1) - s;
 }

 *gc = d / t;
}

/* cubic spline interpolation */

static float _u[WKSP_SIZE];

void spline(float *x, float *y, const size_t n, float *y2)
{
 int i, k, ii, id, sz = n - 1, _sz = n - 2;
 float p, qn, sig, un;
 float *u = NULL;

 if (n < 3) {
  fprintf(stderr, "n is too small in spline().\n");
  exit(EXIT_FAILURE);
 }

 if (sz <= WKSP_SIZE)
  u = _u;

 else {
  u = (float *) (malloc(sz * sizeof(float)));

  if (u == NULL) {
   fprintf(stderr, "Short of memory in spline().\n");
   exit(EXIT_FAILURE);
  }
 }

 y2[0] = u[0] = 0.0;

 for (i = 1; i < sz; i++) {
  ii = i + 1;
  id = i - 1;

  sig = (x[i] - x[id]) / (x[ii] - x[id]);
  p = sig * y2[id] + 2.0;
  y2[i] = (sig - 1.0) / p;
  u[i] = (y[ii] - y[i]) / (x[ii] - x[i]) - (y[i] - y[id]) / (x[i] - x[id]);
  u[i] = (6.0 * u[i] / (x[ii] - x[id]) - sig * u[id]) / p;
 }

 qn = un = 0.0;

 y2[sz] = (un - qn * u[_sz]) / (qn * y2[_sz] + 1.0);

 for (k = _sz; k >= 0; k--)
  y2[k] = y2[k] * y2[k + 1] + u[k];

 if (sz <= WKSP_SIZE)
  return;

 free((char *) (u));
}

float splint(float *xa, float *ya, float *y2a, const size_t n, float x)
{
 int klo = 0, khi = n - 1, k;
 float h, b, a, y;

 if (n < 3) {
  fprintf(stderr, "n is too small in splint().\n");
  exit(EXIT_FAILURE);
 }

 while (khi - klo > 1) {
  k = (khi + klo) >> 1;
  if (xa[k] > x)
   khi = k;
  else
   klo = k;
 }

 h = xa[khi] - xa[klo];

 if (h == 0.0) {
  fprintf(stderr, "Bad xa in spline().\n");
  exit(EXIT_FAILURE);
 }

 a = (xa[khi] - x) / h;
 b = (x - xa[klo]) / h;

 y = a * ya[klo] + b * ya[khi] + (a * (a + 1.0) * (a - 1.0) * y2a[klo] + b * (b + 1.0) * (b -
	 1.0) * y2a[khi]) * h * h / 6.0;

 return y;
}

static void shift3(float *a, float *b, float *c, float *d)
{
 *a = *b;
 *b = *c;
 *c = *d;
}

float brent(float ax, float bx, float cx, float (*f) (float), float tol, float *xmin)
{
 const int MAXIT = 100;
 const double CGOLD = 0.3819660;
 const double ZEPS = 2.22e-16;
 int iter;
 float a, b, d = 0.0, etemp, fu, fv, fw, fx;
 float p, q, r, tol1, tol2, u, v, w, x, xm;
 float e = 0.0;

 a = (ax < cx ? ax : cx);
 b = (ax > cx ? ax : cx);

 x = w = v = bx;
 fw = fv = fx = (*f) (x);

 for (iter = 0; iter < MAXIT; iter++) {
  xm = 0.5 * (a + b);
  tol2 = 2.0 * (tol1 = tol * fabs(x) + ZEPS);

  if (fabs(x - xm) <= (tol2 - 0.5 * (b - a))) {
   *xmin = x;
   return fx;
  }

  if (fabs(e) > tol1) {
   r = (x - w) * (fx - fv);
   q = (x - v) * (fx - fw);
   p = (x - v) * q - (x - w) * r;
   q = 2.0 * (q - r);

   if (q > 0.0)
    p = -p;

   q = fabs(q);
   etemp = e;
   e = d;

   if (fabs(p) >= fabs(0.5 * q * etemp) || p <= q * (a - x) || p >= q * (b - x))
    d = CGOLD * (e = (x >= xm ? a - x : b - x));

   else {
    d = p / q;
    u = x + d;

    if (u - a < tol2 || b - u < tol2)
     d = (xm - x >= 0.0 ? fabs(tol1) : -fabs(tol1));
   }
  }

  else
   d = CGOLD * (e = (x >= xm ? a - x : b - x));

  u = (fabs(d) >= tol1 ? x + d : x + (d >= 0.0 ? fabs(tol1) : -fabs(tol1)));
  fu = (*f) (u);

  if (fu <= fx) {
   if (u >= x)
    a = x;
   else
    b = x;

   shift3(&v, &w, &x, &u);
   shift3(&fv, &fw, &fx, &fu);
  }

  else {
   if (u < x)
    a = u;
   else
    b = u;

   if (fu <= fw || x == w) {
    v = w;
    w = u;
    fv = fw;
    fw = fu;
   }

   else if (fu <= fv || v == x || v == w) {
    v = u;
    fv = fu;
   }
  }
 }

 *xmin = x;

 return fx;
}

float brent_thrdsafe(float ax, float bx, float cx, float (*f) (int, float), float tol, float *xmin, int thrd_id)
{
 const int MAXIT = 100;
 const double CGOLD = 0.3819660;
 const double ZEPS = 2.22e-16;
 int iter;
 float a, b, d = 0.0, etemp, fu, fv, fw, fx;
 float p, q, r, tol1, tol2, u, v, w, x, xm;
 float e = 0.0;

 a = (ax < cx ? ax : cx);
 b = (ax > cx ? ax : cx);

 x = w = v = bx;
 fw = fv = fx = (*f) (thrd_id, x);

 for (iter = 0; iter < MAXIT; iter++) {
  xm = 0.5 * (a + b);
  tol2 = 2.0 * (tol1 = tol * fabs(x) + ZEPS);

  if (fabs(x - xm) <= (tol2 - 0.5 * (b - a))) {
   *xmin = x;
   return fx;
  }

  if (fabs(e) > tol1) {
   r = (x - w) * (fx - fv);
   q = (x - v) * (fx - fw);
   p = (x - v) * q - (x - w) * r;
   q = 2.0 * (q - r);

   if (q > 0.0)
    p = -p;

   q = fabs(q);
   etemp = e;
   e = d;

   if (fabs(p) >= fabs(0.5 * q * etemp) || p <= q * (a - x) || p >= q * (b - x))
    d = CGOLD * (e = (x >= xm ? a - x : b - x));

   else {
    d = p / q;
    u = x + d;

    if (u - a < tol2 || b - u < tol2)
     d = (xm - x >= 0.0 ? fabs(tol1) : -fabs(tol1));
   }
  }

  else
   d = CGOLD * (e = (x >= xm ? a - x : b - x));

  u = (fabs(d) >= tol1 ? x + d : x + (d >= 0.0 ? fabs(tol1) : -fabs(tol1)));
  fu = (*f) (thrd_id, u);

  if (fu <= fx) {
   if (u >= x)
    a = x;
   else
    b = x;

   shift3(&v, &w, &x, &u);
   shift3(&fv, &fw, &fx, &fu);
  }

  else {
   if (u < x)
    a = u;
   else
    b = u;

   if (fu <= fw || x == w) {
    v = w;
    w = u;
    fv = fw;
    fw = fu;
   }

   else if (fu <= fv || v == x || v == w) {
    v = u;
    fv = fu;
   }
  }
 }

 *xmin = x;

 return fx;
}

/* complex */

complex Cadd(complex a, complex b)
{
 complex c;

 c.r = a.r + b.r;
 c.i = a.i + b.i;

 return c;
}

complex Csub(complex a, complex b)
{
 complex c;

 c.r = a.r - b.r;
 c.i = a.i - b.i;

 return c;
}

complex Cmul(complex a, complex b)
{
 complex c;

 c.r = a.r * b.r - a.i * b.i;
 c.i = a.i * b.r + a.r * b.i;

 return c;
}

complex Conj(complex z)
{
 complex c;

 c.r = z.r;
 c.i = -z.i;

 return c;
}

complex Cdiv(complex a, complex b)
{
 complex c;
 float r, den;

 if (fabs(b.r) >= fabs(b.i)) {
  r = b.i / b.r;
  den = b.r + r * b.i;
  c.r = (a.r + r * a.i) / den;
  c.i = (a.i - r * a.r) / den;
 }

 else {
  r = b.r / b.i;
  den = b.i + r * b.r;
  c.r = (a.r * r + a.i) / den;
  c.i = (a.i * r - a.r) / den;
 }

 return c;
}

float Cabs(complex z)
{
 float x, y, r, ans;

 x = fabs(z.r);
 y = fabs(z.i);

 if (x == 0.0)
  ans = y;

 else if (y == 0.0)
  ans = x;

 else if (x > y) {
  r = y / x;
  ans = x * sqrt(1.0 + r * r);
 }

 else {
  r = x / y;
  ans = y * sqrt(1.0 + r * r);
 }

 return ans;
}

complex Csqrt(complex z)
{
 complex c;
 float x, y, w, r;

 if (z.r == 0.0 && z.i == 0.0) {
  c.r = 0.0;
  c.i = 0.0;
 }

 else {
  x = fabs(z.r);
  y = fabs(z.i);

  if (x >= y) {
   r = y / x;
   w = sqrt(x) * sqrt(0.5 * (1.0 + sqrt(1.0 + r * r)));
  }

  else {
   r = x / y;
   w = sqrt(y) * sqrt(0.5 * (r + sqrt(1.0 + r * r)));
  }

  if (z.r >= 0.0) {
   c.r = w;
   c.i = z.i / (2.0 * w);
  }

  else {
   c.i = (z.i >= 0.0) ? w : -w;
   c.r = z.i / (2.0 * c.i);
  }
 }

 return c;
}

complex RCmul(float x, complex a)
{
 complex c;

 c.r = x * a.r;
 c.i = x * a.i;

 return c;
}

complex Cexp(complex z)
{
 complex c;
 float r;

 r = exp(z.r);

 c.r = r * cos(z.i);
 c.i = r * sin(z.i);

 return c;
}

/* double complex */

doublecomplex DCadd(doublecomplex a, doublecomplex b)
{
 doublecomplex c;

 c.r = a.r + b.r;
 c.i = a.i + b.i;

 return c;
}

doublecomplex DCsub(doublecomplex a, doublecomplex b)
{
 doublecomplex c;

 c.r = a.r - b.r;
 c.i = a.i - b.i;

 return c;
}

doublecomplex DCmul(doublecomplex a, doublecomplex b)
{
 doublecomplex c;

 c.r = a.r * b.r - a.i * b.i;
 c.i = a.i * b.r + a.r * b.i;

 return c;
}

doublecomplex DConj(doublecomplex z)
{
 doublecomplex c;

 c.r = z.r;
 c.i = -z.i;

 return c;
}

doublecomplex DCdiv(doublecomplex a, doublecomplex b)
{
 doublecomplex c;
 double r, den;

 if (fabs(b.r) >= fabs(b.i)) {
  r = b.i / b.r;
  den = b.r + r * b.i;
  c.r = (a.r + r * a.i) / den;
  c.i = (a.i - r * a.r) / den;
 }

 else {
  r = b.r / b.i;
  den = b.i + r * b.r;
  c.r = (a.r * r + a.i) / den;
  c.i = (a.i * r - a.r) / den;
 }

 return c;
}

double DCabs(doublecomplex z)
{
 double x, y, r, ans;

 x = fabs(z.r);
 y = fabs(z.i);

 if (x == 0.0)
  ans = y;

 else if (y == 0.0)
  ans = x;

 else if (x > y) {
  r = y / x;
  ans = x * sqrt(1.0 + r * r);
 }

 else {
  r = x / y;
  ans = y * sqrt(1.0 + r * r);
 }

 return ans;
}

doublecomplex DCsqrt(doublecomplex z)
{
 doublecomplex c;
 double x, y, w, r;

 if (z.r == 0.0 && z.i == 0.0) {
  c.r = 0.0;
  c.i = 0.0;
 }

 else {
  x = fabs(z.r);
  y = fabs(z.i);

  if (x >= y) {
   r = y / x;
   w = sqrt(x) * sqrt(0.5 * (1.0 + sqrt(1.0 + r * r)));
  }

  else {
   r = x / y;
   w = sqrt(y) * sqrt(0.5 * (r + sqrt(1.0 + r * r)));
  }

  if (z.r >= 0.0) {
   c.r = w;
   c.i = z.i / (2.0 * w);
  }

  else {
   c.i = (z.i >= 0.0) ? w : -w;
   c.r = z.i / (2.0 * c.i);
  }
 }

 return c;
}

doublecomplex RDCmul(double x, doublecomplex a)
{
 doublecomplex c;

 c.r = x * a.r;
 c.i = x * a.i;

 return c;
}

doublecomplex DCexp(doublecomplex z)
{
 doublecomplex c;
 double r;

 r = exp(z.r);

 c.r = r * cos(z.i);
 c.i = r * sin(z.i);

 return c;
}

/* hyper complex */

hypercomplex HCadd(hypercomplex a, hypercomplex b)
{
 hypercomplex c;

 c.x = Cadd(a.x, b.x);
 c.y = Cadd(a.y, b.y);

 return c;
}

hypercomplex HCsub(hypercomplex a, hypercomplex b)
{
 hypercomplex c;

 c.x = Csub(a.x, b.x);
 c.y = Csub(a.y, b.y);

 return c;
}

hypercomplex HCmul(hypercomplex a, hypercomplex b)
{
 hypercomplex c;

 c.x = Cmul(a.x, b.x);
 c.y = Cmul(a.y, b.y);

 return c;
}

hypercomplex HConj(hypercomplex z)
{
 hypercomplex c;

 c.x = Conj(z.x);
 c.y = Conj(z.y);

 return c;
}

hypercomplex HCtp(hypercomplex z)
{
 hypercomplex c;

 c.x = z.y;
 c.y = z.x;

 return c;
}

/* quaternion */

quaternion Qadd(quaternion a, quaternion b)
{
 quaternion c;

 c.r = a.r + b.r;
 c.i = a.i + b.i;
 c.j = a.j + b.j;
 c.k = a.k + b.k;

 return c;
}

quaternion Qsub(quaternion a, quaternion b)
{
 quaternion c;

 c.r = a.r - b.r;
 c.i = a.i - b.i;
 c.j = a.j - b.j;
 c.k = a.k - b.k;

 return c;
}

quaternion Qmul(quaternion a, quaternion b)
{
 quaternion c;

 c.r = a.r * b.r - a.i * b.i - a.j * b.j - a.k - b.k;
 c.i = a.i * b.r + a.r * b.i + a.j * b.k - a.k * b.j;
 c.j = a.j * b.r + a.r * b.j + a.k * b.i - a.i * b.k;
 c.k = a.k * b.r + a.r * b.k + a.i * b.j - a.j * b.i;

 return c;
}

quaternion Qonj(quaternion z)
{
 quaternion c;

 c.r = z.r;
 c.i = -z.i;
 c.j = -z.j;
 c.k = -z.k;

 return c;
}
